# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc4bmi/bmi.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12grpc4bmi/bmi.proto\x12\x03\x62mi\"\x07\n\x05\x45mpty\"(\n\x11InitializeRequest\x12\x13\n\x0b\x63onfig_file\x18\x01 \x01(\t\"(\n\x18GetComponentNameResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\"$\n\x13GetVarNamesResponse\x12\r\n\x05names\x18\x01 \x03(\t\"%\n\x14GetTimeUnitsResponse\x12\r\n\x05units\x18\x01 \x01(\t\"\'\n\x13GetTimeStepResponse\x12\x10\n\x08interval\x18\x01 \x01(\x01\"\x1f\n\x0fGetTimeResponse\x12\x0c\n\x04time\x18\x01 \x01(\x01\"\x1d\n\rGetVarRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"%\n\x12GetVarGridResponse\x12\x0f\n\x07grid_id\x18\x01 \x01(\x05\"\"\n\x12GetVarTypeResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\"&\n\x16GetVarItemSizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"$\n\x13GetVarUnitsResponse\x12\r\n\x05units\x18\x01 \x01(\t\"&\n\x14GetVarNBytesResponse\x12\x0e\n\x06nbytes\x18\x01 \x01(\x03\"z\n\x16GetVarLocationResponse\x12\x36\n\x08location\x18\x01 \x01(\x0e\x32$.bmi.GetVarLocationResponse.Location\"(\n\x08Location\x12\x08\n\x04NODE\x10\x00\x12\x08\n\x04\x45\x44GE\x10\x01\x12\x08\n\x04\x46\x41\x43\x45\x10\x02\"%\n\x0fIntArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x03\x42\x02\x10\x01\"\'\n\x11\x46loatArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x02\x42\x02\x10\x01\"(\n\x12\x44oubleArrayMessage\x12\x12\n\x06values\x18\x01 \x03(\x01\x42\x02\x10\x01\"\xaa\x01\n\x10GetValueResponse\x12*\n\nvalues_int\x18\x01 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x02 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x03 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"=\n\x18GetValueAtIndicesRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x07indices\x18\x02 \x03(\x03\x42\x02\x10\x01\"\xb3\x01\n\x19GetValueAtIndicesResponse\x12*\n\nvalues_int\x18\x01 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x02 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x03 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"\xb7\x01\n\x0fSetValueRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12*\n\nvalues_int\x18\x02 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x03 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x04 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"/\n\x12SetValuePtrRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03ref\x18\x02 \x01(\x03\"\xd5\x01\n\x18SetValueAtIndicesRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x07indices\x18\x02 \x03(\x03\x42\x02\x10\x01\x12*\n\nvalues_int\x18\x03 \x01(\x0b\x32\x14.bmi.IntArrayMessageH\x00\x12.\n\x0cvalues_float\x18\x04 \x01(\x0b\x32\x16.bmi.FloatArrayMessageH\x00\x12\x30\n\rvalues_double\x18\x05 \x01(\x0b\x32\x17.bmi.DoubleArrayMessageH\x00\x42\x08\n\x06values\"\x1e\n\x0bGridRequest\x12\x0f\n\x07grid_id\x18\x01 \x01(\x03\"#\n\x13GetGridSizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"#\n\x13GetGridRankResponse\x12\x0c\n\x04rank\x18\x01 \x01(\x03\"#\n\x13GetGridTypeResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\")\n\x14GetGridShapeResponse\x12\x11\n\x05shape\x18\x01 \x03(\x03\x42\x02\x10\x01\"-\n\x16GetGridSpacingResponse\x12\x13\n\x07spacing\x18\x01 \x03(\x01\x42\x02\x10\x01\"+\n\x15GetGridOriginResponse\x12\x12\n\x06origin\x18\x01 \x03(\x01\x42\x02\x10\x01\"0\n\x15GetGridPointsResponse\x12\x17\n\x0b\x63oordinates\x18\x01 \x03(\x01\x42\x02\x10\x01\"!\n\x10GetCountResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\"2\n\x18GetGridEdgeNodesResponse\x12\x16\n\nedge_nodes\x18\x01 \x03(\x03\x42\x02\x10\x01\"2\n\x18GetGridFaceEdgesResponse\x12\x16\n\nface_edges\x18\x01 \x03(\x03\x42\x02\x10\x01\"2\n\x18GetGridFaceNodesResponse\x12\x16\n\nface_nodes\x18\x01 \x03(\x03\x42\x02\x10\x01\"9\n\x1bGetGridNodesPerFaceResponse\x12\x1a\n\x0enodes_per_face\x18\x01 \x03(\x03\x42\x02\x10\x01\x32\x8a\x13\n\nBmiService\x12\x32\n\ninitialize\x12\x16.bmi.InitializeRequest\x1a\n.bmi.Empty\"\x00\x12\"\n\x06update\x12\n.bmi.Empty\x1a\n.bmi.Empty\"\x00\x12\x31\n\x0bupdateUntil\x12\x14.bmi.GetTimeResponse\x1a\n.bmi.Empty\"\x00\x12$\n\x08\x66inalize\x12\n.bmi.Empty\x1a\n.bmi.Empty\"\x00\x12?\n\x10getComponentName\x12\n.bmi.Empty\x1a\x1d.bmi.GetComponentNameResponse\"\x00\x12\x38\n\x11getInputItemCount\x12\n.bmi.Empty\x1a\x15.bmi.GetCountResponse\"\x00\x12\x39\n\x12getOutputItemCount\x12\n.bmi.Empty\x1a\x15.bmi.GetCountResponse\"\x00\x12:\n\x10getInputVarNames\x12\n.bmi.Empty\x1a\x18.bmi.GetVarNamesResponse\"\x00\x12;\n\x11getOutputVarNames\x12\n.bmi.Empty\x1a\x18.bmi.GetVarNamesResponse\"\x00\x12\x37\n\x0cgetTimeUnits\x12\n.bmi.Empty\x1a\x19.bmi.GetTimeUnitsResponse\"\x00\x12\x35\n\x0bgetTimeStep\x12\n.bmi.Empty\x1a\x18.bmi.GetTimeStepResponse\"\x00\x12\x34\n\x0egetCurrentTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12\x32\n\x0cgetStartTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12\x30\n\ngetEndTime\x12\n.bmi.Empty\x1a\x14.bmi.GetTimeResponse\"\x00\x12;\n\ngetVarGrid\x12\x12.bmi.GetVarRequest\x1a\x17.bmi.GetVarGridResponse\"\x00\x12;\n\ngetVarType\x12\x12.bmi.GetVarRequest\x1a\x17.bmi.GetVarTypeResponse\"\x00\x12\x43\n\x0egetVarItemSize\x12\x12.bmi.GetVarRequest\x1a\x1b.bmi.GetVarItemSizeResponse\"\x00\x12=\n\x0bgetVarUnits\x12\x12.bmi.GetVarRequest\x1a\x18.bmi.GetVarUnitsResponse\"\x00\x12?\n\x0cgetVarNBytes\x12\x12.bmi.GetVarRequest\x1a\x19.bmi.GetVarNBytesResponse\"\x00\x12\x43\n\x0egetVarLocation\x12\x12.bmi.GetVarRequest\x1a\x1b.bmi.GetVarLocationResponse\"\x00\x12\x37\n\x08getValue\x12\x12.bmi.GetVarRequest\x1a\x15.bmi.GetValueResponse\"\x00\x12T\n\x11getValueAtIndices\x12\x1d.bmi.GetValueAtIndicesRequest\x1a\x1e.bmi.GetValueAtIndicesResponse\"\x00\x12.\n\x08setValue\x12\x14.bmi.SetValueRequest\x1a\n.bmi.Empty\"\x00\x12@\n\x11setValueAtIndices\x12\x1d.bmi.SetValueAtIndicesRequest\x1a\n.bmi.Empty\"\x00\x12;\n\x0bgetGridSize\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridSizeResponse\"\x00\x12;\n\x0bgetGridType\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridTypeResponse\"\x00\x12;\n\x0bgetGridRank\x12\x10.bmi.GridRequest\x1a\x18.bmi.GetGridRankResponse\"\x00\x12=\n\x0cgetGridShape\x12\x10.bmi.GridRequest\x1a\x19.bmi.GetGridShapeResponse\"\x00\x12\x41\n\x0egetGridSpacing\x12\x10.bmi.GridRequest\x1a\x1b.bmi.GetGridSpacingResponse\"\x00\x12?\n\rgetGridOrigin\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridOriginResponse\"\x00\x12:\n\x08getGridX\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12:\n\x08getGridY\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12:\n\x08getGridZ\x12\x10.bmi.GridRequest\x1a\x1a.bmi.GetGridPointsResponse\"\x00\x12=\n\x10getGridNodeCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12=\n\x10getGridEdgeCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12=\n\x10getGridFaceCount\x12\x10.bmi.GridRequest\x1a\x15.bmi.GetCountResponse\"\x00\x12\x45\n\x10getGridEdgeNodes\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridEdgeNodesResponse\"\x00\x12\x45\n\x10getGridFaceNodes\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridFaceNodesResponse\"\x00\x12\x45\n\x10getGridFaceEdges\x12\x10.bmi.GridRequest\x1a\x1d.bmi.GetGridFaceEdgesResponse\"\x00\x12K\n\x13getGridNodesPerFace\x12\x10.bmi.GridRequest\x1a .bmi.GetGridNodesPerFaceResponse\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc4bmi.bmi_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INTARRAYMESSAGE.fields_by_name['values']._options = None
  _INTARRAYMESSAGE.fields_by_name['values']._serialized_options = b'\020\001'
  _FLOATARRAYMESSAGE.fields_by_name['values']._options = None
  _FLOATARRAYMESSAGE.fields_by_name['values']._serialized_options = b'\020\001'
  _DOUBLEARRAYMESSAGE.fields_by_name['values']._options = None
  _DOUBLEARRAYMESSAGE.fields_by_name['values']._serialized_options = b'\020\001'
  _GETVALUEATINDICESREQUEST.fields_by_name['indices']._options = None
  _GETVALUEATINDICESREQUEST.fields_by_name['indices']._serialized_options = b'\020\001'
  _SETVALUEATINDICESREQUEST.fields_by_name['indices']._options = None
  _SETVALUEATINDICESREQUEST.fields_by_name['indices']._serialized_options = b'\020\001'
  _GETGRIDSHAPERESPONSE.fields_by_name['shape']._options = None
  _GETGRIDSHAPERESPONSE.fields_by_name['shape']._serialized_options = b'\020\001'
  _GETGRIDSPACINGRESPONSE.fields_by_name['spacing']._options = None
  _GETGRIDSPACINGRESPONSE.fields_by_name['spacing']._serialized_options = b'\020\001'
  _GETGRIDORIGINRESPONSE.fields_by_name['origin']._options = None
  _GETGRIDORIGINRESPONSE.fields_by_name['origin']._serialized_options = b'\020\001'
  _GETGRIDPOINTSRESPONSE.fields_by_name['coordinates']._options = None
  _GETGRIDPOINTSRESPONSE.fields_by_name['coordinates']._serialized_options = b'\020\001'
  _GETGRIDEDGENODESRESPONSE.fields_by_name['edge_nodes']._options = None
  _GETGRIDEDGENODESRESPONSE.fields_by_name['edge_nodes']._serialized_options = b'\020\001'
  _GETGRIDFACEEDGESRESPONSE.fields_by_name['face_edges']._options = None
  _GETGRIDFACEEDGESRESPONSE.fields_by_name['face_edges']._serialized_options = b'\020\001'
  _GETGRIDFACENODESRESPONSE.fields_by_name['face_nodes']._options = None
  _GETGRIDFACENODESRESPONSE.fields_by_name['face_nodes']._serialized_options = b'\020\001'
  _GETGRIDNODESPERFACERESPONSE.fields_by_name['nodes_per_face']._options = None
  _GETGRIDNODESPERFACERESPONSE.fields_by_name['nodes_per_face']._serialized_options = b'\020\001'
  _EMPTY._serialized_start=27
  _EMPTY._serialized_end=34
  _INITIALIZEREQUEST._serialized_start=36
  _INITIALIZEREQUEST._serialized_end=76
  _GETCOMPONENTNAMERESPONSE._serialized_start=78
  _GETCOMPONENTNAMERESPONSE._serialized_end=118
  _GETVARNAMESRESPONSE._serialized_start=120
  _GETVARNAMESRESPONSE._serialized_end=156
  _GETTIMEUNITSRESPONSE._serialized_start=158
  _GETTIMEUNITSRESPONSE._serialized_end=195
  _GETTIMESTEPRESPONSE._serialized_start=197
  _GETTIMESTEPRESPONSE._serialized_end=236
  _GETTIMERESPONSE._serialized_start=238
  _GETTIMERESPONSE._serialized_end=269
  _GETVARREQUEST._serialized_start=271
  _GETVARREQUEST._serialized_end=300
  _GETVARGRIDRESPONSE._serialized_start=302
  _GETVARGRIDRESPONSE._serialized_end=339
  _GETVARTYPERESPONSE._serialized_start=341
  _GETVARTYPERESPONSE._serialized_end=375
  _GETVARITEMSIZERESPONSE._serialized_start=377
  _GETVARITEMSIZERESPONSE._serialized_end=415
  _GETVARUNITSRESPONSE._serialized_start=417
  _GETVARUNITSRESPONSE._serialized_end=453
  _GETVARNBYTESRESPONSE._serialized_start=455
  _GETVARNBYTESRESPONSE._serialized_end=493
  _GETVARLOCATIONRESPONSE._serialized_start=495
  _GETVARLOCATIONRESPONSE._serialized_end=617
  _GETVARLOCATIONRESPONSE_LOCATION._serialized_start=577
  _GETVARLOCATIONRESPONSE_LOCATION._serialized_end=617
  _INTARRAYMESSAGE._serialized_start=619
  _INTARRAYMESSAGE._serialized_end=656
  _FLOATARRAYMESSAGE._serialized_start=658
  _FLOATARRAYMESSAGE._serialized_end=697
  _DOUBLEARRAYMESSAGE._serialized_start=699
  _DOUBLEARRAYMESSAGE._serialized_end=739
  _GETVALUERESPONSE._serialized_start=742
  _GETVALUERESPONSE._serialized_end=912
  _GETVALUEATINDICESREQUEST._serialized_start=914
  _GETVALUEATINDICESREQUEST._serialized_end=975
  _GETVALUEATINDICESRESPONSE._serialized_start=978
  _GETVALUEATINDICESRESPONSE._serialized_end=1157
  _SETVALUEREQUEST._serialized_start=1160
  _SETVALUEREQUEST._serialized_end=1343
  _SETVALUEPTRREQUEST._serialized_start=1345
  _SETVALUEPTRREQUEST._serialized_end=1392
  _SETVALUEATINDICESREQUEST._serialized_start=1395
  _SETVALUEATINDICESREQUEST._serialized_end=1608
  _GRIDREQUEST._serialized_start=1610
  _GRIDREQUEST._serialized_end=1640
  _GETGRIDSIZERESPONSE._serialized_start=1642
  _GETGRIDSIZERESPONSE._serialized_end=1677
  _GETGRIDRANKRESPONSE._serialized_start=1679
  _GETGRIDRANKRESPONSE._serialized_end=1714
  _GETGRIDTYPERESPONSE._serialized_start=1716
  _GETGRIDTYPERESPONSE._serialized_end=1751
  _GETGRIDSHAPERESPONSE._serialized_start=1753
  _GETGRIDSHAPERESPONSE._serialized_end=1794
  _GETGRIDSPACINGRESPONSE._serialized_start=1796
  _GETGRIDSPACINGRESPONSE._serialized_end=1841
  _GETGRIDORIGINRESPONSE._serialized_start=1843
  _GETGRIDORIGINRESPONSE._serialized_end=1886
  _GETGRIDPOINTSRESPONSE._serialized_start=1888
  _GETGRIDPOINTSRESPONSE._serialized_end=1936
  _GETCOUNTRESPONSE._serialized_start=1938
  _GETCOUNTRESPONSE._serialized_end=1971
  _GETGRIDEDGENODESRESPONSE._serialized_start=1973
  _GETGRIDEDGENODESRESPONSE._serialized_end=2023
  _GETGRIDFACEEDGESRESPONSE._serialized_start=2025
  _GETGRIDFACEEDGESRESPONSE._serialized_end=2075
  _GETGRIDFACENODESRESPONSE._serialized_start=2077
  _GETGRIDFACENODESRESPONSE._serialized_end=2127
  _GETGRIDNODESPERFACERESPONSE._serialized_start=2129
  _GETGRIDNODESPERFACERESPONSE._serialized_end=2186
  _BMISERVICE._serialized_start=2189
  _BMISERVICE._serialized_end=4631
# @@protoc_insertion_point(module_scope)
